// Stefanie Miller
// CPS 406
// November 4, 1999

// Assignment 3

// Graph.java
//
// The Graph class creates the graphical representation of functions f0 
// through f3.  It does this by drawing on itself (being a canvas).
//
// API of class Graph
// 
// public class Graph extends Canvas {
//   Graph (int w, int h);
//   public void zoom (double n);
//   public void pain (Graphics g, int index);
// }
//


import java.awt.*;         // standard graphics package
import java.io.*;          // standard input/output package includes file io


public class Graph extends Canvas {

    //=========================================================================
    // Private Instance variables
    //=========================================================================
    protected static final int spaces = 10;    // length of tick marks
    protected static int xmin = -10;           // min x value on the graph
    protected static int xmax =  10;           // max x value on the graph
    protected static int ymin = -10;           // min y value on the graph
    protected static int ymax =  10;           // max y value on the graph
    protected static int width;                // width of the canvas
    protected static int height;               // height of the canvas
    protected static double scale;             // scale of the graph

    //=========================================================================
    // Constructor for Graph class
    //=========================================================================
    Graph (int w, int h) {
	
	// intialize width and height
	width  = w;
	height = h;
	
	// resize the canvas to be width by height
	this.setBounds(0, 0, width, height);

    }

    //=========================================================================
    // zoom: zooms in/out by the double value n
    //=========================================================================
    public void zoom (double n) {

	int nxmin = (int)(xmin * n);    
	int nxmax = (int)(xmax * n);
        int nymin = (int)(ymin * n);
        int nymax = (int)(ymax * n);

	// if the new min and max values are not out of range, reset them
	if (Math.abs(nxmin) > 0) xmin = nxmin;
	if (Math.abs(nymin) > 0) ymin = nymin;
	if (Math.abs(nxmax) > 0) xmax = nxmax;
	if (Math.abs(nymax) > 0) ymax = nymax;
    }

    //=========================================================================
    // paint: redraws the graph specified by the integer, index in the 
    //        graphics context g
    //======================================================================== 
    public void paint(Graphics g, int index) {

	// erase old graph
	g.setColor(Color.white);
	g.fillRect(0, 0, width+15, height+15);

	// set font
	g.setFont(new Font(g.getFont().getName(), g.getFont().getStyle(), 
			   g.getFont().getSize()));

	// draw axis
	g.setColor(Color.black);
	g.drawLine(width/2, 0, width/2, height);   // y-axis
	g.drawLine(0, height/2, width, height/2);  // x-axis

        // set origin
        g.translate(width/2, height/2);

        // calculate scale factor
        scale = (double)(Math.abs(xmin) + Math.abs(xmax)) / width;

        int max = (int)(xmax/scale);
        int min = (int)(xmin/scale);
	
	// write the x scale on the x-axis
	g.setColor(Color.red);
        g.drawString(""+xmin, min, 12);
        g.drawString(""+xmax, max, 12);
 
        // draw actual function
        g.setColor(Color.blue);
	
	// calculate each point in the curve and draw it
	int j;
        for (int i = min; i <= max; i++) {
	    
	    if (index == 0)
		j = f0(i, scale);
	    else if (index == 1)
		j = f1(i, scale);    
	    else if (index == 2)
		j = f2(i, scale);    
	    else j = f3(i, scale);    

	    // reverse the sign of j because screen coordinates are the
	    // opposite of cartesian coordinates for the y-axis
	    j *= -1;

	    // if the point (i,j) is in range, draw it
	    int relj = (int)(j * scale);
	    if ((relj > ymin) && (relj < ymax)) {
		g.drawLine(i, j, i, j);
	    }
        }    
    }

    //=========================================================================
    // f0: the first function in the list
    //=========================================================================
    protected int f0 (int x, double scale) {
	double nx = x * scale;
	double ans = nx + 1;
        return (int)(ans / scale);
    }

    //=========================================================================
    // f1: the second function in the list
    //=========================================================================
    protected int f1 (int x, double scale) {
	double nx = x * scale;
	double ans = Math.pow(nx, 2) + nx + 1;	
        return (int)(ans/scale);
    }   

    //=========================================================================
    // f2: the third function in the list
    //=========================================================================
    protected int f2 (int x, double scale) {
        double nx = x * scale;
        double ans = Math.pow(nx, 3) + Math.pow(nx, 2) + nx + 1;
        return (int)(ans/scale);
    }  

    //=========================================================================
    // f3: the third function in the list
    //=========================================================================
    protected int f3 (int x, double scale) {
        double nx = x * scale;
        double ans = Math.sin(nx);
        return (int)(ans/scale);
    }    
}   











